#!/bin/bash

# import utilities
HERE=$(cd $(dirname $BASH_SOURCE) && pwd)
source $HERE/testutil.sh

colineard tx colinear new-auction asdf asdf 200 token $(now + 3500) \
    -y --from bob \
    | expect_fail "Can't create auction below min lease period" 

colineard tx colinear new-auction asdf asdf 200 token $(now + 8200000) \
    -y --from bob \
    | expect_fail "Can't create auction above max lease period" 

before=$(get_balance $BOB token)

colineard tx colinear new-auction asdf asdf 200 token $(now + 3700) \
    -y --from bob \
    | expect_success "New auction is created" 

get_balance $BOB token | expect_change -200 $before "Change in Bob's balance"

colineard tx colinear new-bid $(get_last_auction_index) 100 \
    -y --from alice \
    | expect_success "New bid is correctly created"

colineard tx colinear new-bid $(get_last_auction_index) 90 \
    -y --from alice \
    | expect_success "Can bid below price ceiling"

colineard tx colinear new-bid $(get_last_auction_index) 100 \
    -y --from alice \
    | expect_fail "Can't create duplicate bids"

colineard tx colinear new-bid $(get_next_auction_index) 100 \
    -y --from alice \
    | expect_fail "Can't create bids for nonexistent auctions"

colineard tx colinear new-bid $(get_last_auction_index) 600 \
    -y --from alice \
    | expect_fail "Can't bid over price ceiling"

colineard tx colinear new-bid $(get_last_auction_index) 0 \
    -y --from alice \
    | expect_fail "Can't bid 0 tokens"

colineard q colinear auction-bids $(get_last_auction_index) \
    | jq -M ".bids | length" \
    | assert_eq 2 "Number of auction bids"

before=$(get_balance $BOB token)

# Was experiencing issues with bob's balance not updating, probably
# because we weren't waiting long enough
log_info "Waiting 12s until auction expires..."
sleep 12

colineard tx colinear new-bid $(get_last_auction_index) 50 \
    -y --from alice \
    | expect_fail "Can't add bids after auction expiry"

colineard q colinear show-auction $(get_last_auction_index) \
    | jq -rM ".auction.best.amount" \
    | assert_eq 90 "Remaining vested amount from finalized top bid"

get_balance $BOB token | expect_change 110 $before "Change in Bob's balance"
